/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api;

import java.io.File;
import java.util.ArrayList;

import javax.json.JsonArray;
import javax.json.JsonObject;

import com.advancednavigation.kinematica.api.data.Configuration;
import com.advancednavigation.kinematica.api.data.DataSet;
import com.advancednavigation.kinematica.api.data.Configuration.DeviceId;
import com.advancednavigation.kinematica.api.json.DataSetJSON;
import com.advancednavigation.kinematica.api.json.KinematicaFileJSON;
import com.advancednavigation.kinematica.api.json.LogJSON;
import com.advancednavigation.kinematica.api.json.StatusJSON;
import com.advancednavigation.kinematica.api.util.PostMultipartData;
import com.advancednavigation.kinematica.api.util.PostUrlEncodedData;
import com.advancednavigation.kinematica.api.util.Requests;

public class KinematicaApi implements KinematicaApiInterface
{
	// Kinematica API v1
	private static String API_VERSION = "1.2";
	private String API_KEY = "";

	private static String SERVER = "https://hq.advancednavigation.com.au";

	@Override
	public void setApiKey(String apiKey)
	{
		API_KEY = apiKey;
	}

	@Override
	public ArrayList<DataSetJSON> allDataSets() throws Exception
	{
		// Send get request
		String json = Requests.get(SERVER + "/kinematica/api/alldatasets?v=" + API_VERSION + "&apiKey=" + API_KEY);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);
			if (object.getString("success").equals("true"))
			{
				ArrayList<DataSetJSON> dataSets = new ArrayList<DataSetJSON>();
				JsonArray dataSetsJson = object.getJsonArray("datasets");

				for (int i = 0; i < dataSetsJson.size(); i++)
				{
					JsonObject dataSetJson = dataSetsJson.getJsonObject(i);
					dataSets.add(DataSetJSON.fromJson(dataSetJson.toString()));
				}

				return dataSets;
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public DataSetJSON newDataSet(DataSet dataSet) throws Exception
	{
		// Create post data
		PostMultipartData postData = new PostMultipartData();
		postData.addString("v", API_VERSION);
		postData.addString("apiKey", API_KEY);

		if (dataSet.dataSetName != null) postData.addString("name", dataSet.dataSetName);

		for (int i = 0; i < dataSet.primaryFiles.size(); i++)
		{
			postData.addFile("primaryFiles[" + i + "]", dataSet.primaryFiles.get(i));
		}

		for (int i = 0; i < dataSet.secondaryFiles.size(); i++)
		{
			postData.addFile("secondaryFiles[" + i + "]", dataSet.secondaryFiles.get(i));
		}

		postData.addString("baseStationSource", "" + dataSet.baseStationSource);
		if (dataSet.baseStationSource == DataSet.BASE_STATION_SOURCE_MANUAL)
		{
			for (int i = 0; i < dataSet.baseStationFiles.size(); i++)
			{
				postData.addFile("baseStationFiles[" + i + "]", dataSet.baseStationFiles.get(i));
			}
		}

		postData.addString("ephemerisSource", "" + dataSet.ephemerisSource);
		if (dataSet.ephemerisSource == DataSet.EPHEMERIS_SOURCE_SUPPLEMENT)
		{
			for (int i = 0; i < dataSet.ephemerisFiles.size(); i++)
			{
				postData.addFile("ephemerisFiles[" + i + "]", dataSet.ephemerisFiles.get(i));
			}
		}

		postData.addString("sendEmail", "" + dataSet.sendEmail);

		// Post request
		String json = Requests.postMultipart(SERVER + "/kinematica/api/newdataset", postData);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				JsonObject dataSetJson = object.getJsonObject("dataset");

				return DataSetJSON.fromJson(dataSetJson.toString());
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public ArrayList<String> antennaOptions() throws Exception
	{
		// Send get request
		String json = Requests.get(SERVER + "/kinematica/api/antennaoptions?v=" + API_VERSION + "&apiKey=" + API_KEY);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				ArrayList<String> options = new ArrayList<String>();
				JsonArray optionsJson = object.getJsonArray("options");

				for (int i = 0; i < optionsJson.size(); i++)
				{
					String option = optionsJson.getString(i);
					options.add(option);
				}

				return options;
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}
	}

	@Override
	public boolean editConfiguration(int id, Configuration configuration) throws Exception
	{
		// Create post data
		PostUrlEncodedData postData = new PostUrlEncodedData();
		postData.addString("v", API_VERSION);
		postData.addString("apiKey", API_KEY);
		postData.addString("id", "" + id);

		if (configuration.deviceId != DeviceId.DEFAULT) postData.addString("deviceId", "" + configuration.deviceId.ordinal());

		postData.addString("baseStationAntenna", configuration.baseStationAntenna);
		postData.addString("roverAntenna", configuration.roverAntenna);
		postData.addString("gnssOffsetSource", "" + configuration.gnssOffsetSource);
		postData.addString("odometerOffsetSource", "" + configuration.odometerOffsetSource);
		postData.addString("dualAntennaOffsetSource", "" + configuration.dualAntennaOffsetSource);
		postData.addString("referencePointOffsetSource", "" + configuration.referencePointOffsetSource);

		if (configuration.gnssOffsetSource == Configuration.MANUAL_SOURCE)
		{
			postData.addString("gnssOffset[0]", "" + configuration.gnssOffset[0]);
			postData.addString("gnssOffset[1]", "" + configuration.gnssOffset[1]);
			postData.addString("gnssOffset[2]", "" + configuration.gnssOffset[2]);
		}

		if (configuration.odometerOffsetSource == Configuration.MANUAL_SOURCE)
		{
			postData.addString("odometerOffset[0]", "" + configuration.odometerOffset[0]);
			postData.addString("odometerOffset[1]", "" + configuration.odometerOffset[1]);
			postData.addString("odometerOffset[2]", "" + configuration.odometerOffset[2]);
		}

		if (configuration.dualAntennaOffsetSource == Configuration.MANUAL_SOURCE)
		{
			postData.addString("dualAntennaOffsetType", "" + configuration.dualAntennaOffsetType);

			if (configuration.dualAntennaOffsetType == Configuration.MANUAL_OFFSET)
			{
				postData.addString("dualAntennaManualOffset[0]", "" + configuration.dualAntennaManualOffset[0]);
				postData.addString("dualAntennaManualOffset[1]", "" + configuration.dualAntennaManualOffset[1]);
				postData.addString("dualAntennaManualOffset[2]", "" + configuration.dualAntennaManualOffset[2]);
			}
			else if (configuration.dualAntennaOffsetType == Configuration.AUTOMATIC_OFFSET)
			{
				postData.addString("dualAntennaAutomaticOffset", "" + configuration.dualAntennaAutomaticOffset);
			}
		}

		if (configuration.referencePointOffsetSource == Configuration.MANUAL_SOURCE)
		{
			postData.addString("referencePointOffset[0]", "" + configuration.referencePointOffset[0]);
			postData.addString("referencePointOffset[1]", "" + configuration.referencePointOffset[1]);
			postData.addString("referencePointOffset[2]", "" + configuration.referencePointOffset[2]);
		}

		// Post request
		String json = Requests.postUrlEncoded(SERVER + "/kinematica/api/editconfiguration", postData);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				return true;
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}
	}

	@Override
	public boolean processDataSet(int id) throws Exception
	{
		// Create post data
		PostUrlEncodedData postData = new PostUrlEncodedData();
		postData.addString("v", API_VERSION);
		postData.addString("apiKey", API_KEY);
		postData.addString("id", "" + id);

		// Post request
		String json = Requests.postUrlEncoded(SERVER + "/kinematica/api/processdataset", postData);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				return true;
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public StatusJSON dataSetStatus(int id) throws Exception
	{
		// Send get request
		String json = Requests.get(SERVER + "/kinematica/api/datasetstatus?v=" + API_VERSION + "&apiKey=" + API_KEY + "&id=" + id);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				JsonObject statusJson = object.getJsonObject("status");
				return StatusJSON.fromJson(statusJson.toString());
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public ArrayList<LogJSON> dataSetLogsAfterTime(int id, long time) throws Exception
	{
		// Send get request
		String json = Requests.get(SERVER + "/kinematica/api/datasetlogs?v=" + API_VERSION + "&apiKey=" + API_KEY + "&id=" + id + "&time=" + time);

		// Read response
		try
		{
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("true"))
			{
				ArrayList<LogJSON> logs = new ArrayList<LogJSON>();
				JsonArray logsJson = object.getJsonArray("logs");

				for (int i = 0; i < logsJson.size(); i++)
				{
					JsonObject logJson = logsJson.getJsonObject(i);
					logs.add(LogJSON.fromJson(logJson.toString()));
				}

				return logs;
			}
			else
			{
				throw new Exception(object.getString("message"));
			}
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public ArrayList<KinematicaFileJSON> dataSetFiles(int id) throws Exception
	{
		// Send get request
		String json = Requests.get(SERVER + "/kinematica/api/datasetfiles?v=" + API_VERSION + "&apiKey=" + API_KEY + "&id=" + id);

		// Read response
		try
		{
			ArrayList<KinematicaFileJSON> files = new ArrayList<KinematicaFileJSON>();
			JsonObject object = Requests.parseJson(json);

			if (object.getString("success").equals("false"))
			{
				throw new Exception(object.getString("message"));
			}

			JsonArray userFiles = object.getJsonArray("userFiles");
			for (int i = 0; i < userFiles.size(); i++)
			{
				JsonObject userFile = userFiles.getJsonObject(i);
				files.add(KinematicaFileJSON.fromJson(userFile.toString()));
			}

			JsonArray automaticFiles = object.getJsonArray("automaticFiles");
			for (int i = 0; i < automaticFiles.size(); i++)
			{
				JsonObject automaticFile = automaticFiles.getJsonObject(i);
				files.add(KinematicaFileJSON.fromJson(automaticFile.toString()));
			}

			return files;
		}
		catch (NullPointerException e)
		{
			throw new Exception("Unknown error");
		}

	}

	@Override
	public File downloadDataSetFile(int id, String filename, File directory) throws Exception
	{
		// Create directory
		directory.mkdirs();
		File file = new File(directory.getAbsolutePath() + File.separatorChar + filename);

		// Send get request
		String json = Requests.getDownload(SERVER + "/kinematica/api/downloadfile?v=" + API_VERSION + "&apiKey=" + API_KEY + "&id=" + id + "&filename=" + filename, file);

		// Check response
		if (json != "")
		{
			JsonObject object = Requests.parseJson(json);

			try
			{
				throw new Exception(object.getString("message"));
			}
			catch (NullPointerException e)
			{
				throw new Exception("Unknown error");
			}
		}
		return file;
	}

}
