/****************************************************************/
/*                                                              */
/*            Advanced Navigation Kinematica API v1.2           */
/*              Copyright 2018, Advanced Navigation             */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2018 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.kinematica.api.example;

import java.io.File;
import java.util.ArrayList;

import com.advancednavigation.kinematica.api.json.DataSetJSON;
import com.advancednavigation.kinematica.api.json.KinematicaFileJSON;
import com.advancednavigation.kinematica.api.json.LogJSON;
import com.advancednavigation.kinematica.api.json.StatusJSON;
import com.advancednavigation.kinematica.api.KinematicaApi;
import com.advancednavigation.kinematica.api.data.Configuration;
import com.advancednavigation.kinematica.api.data.DataSet;
import com.advancednavigation.kinematica.api.data.Configuration.DeviceId;

public class APIExample
{
	// Modify the following
	public static String API_KEY = "MODIFY_THIS";

	// Download directory for post processed files
	public static String DOWNLOAD_DIRECTORY = "C:\\tmp";

	// Primary rover file
	public static String PRIMARY_FILE = "C:\\Desktop\\kinematica\\log.anpp";

	// The following are optional
	public static String SECONDARY_FILE      = "C:\\Desktop\\kinematica\\log.anpp";
	public static String BASE_STATION_FILE_1 = "C:\\Desktop\\kinematica\\SYDN111D30.17d.Z";
	public static String EPHEMERIS_FILE_1    = "C:\\Desktop\\kinematica\\brdc1100.17g.Z";
	public static String EPHEMERIS_FILE_2    = "C:\\Desktop\\kinematica\\brdc1100.17n.Z";
	public static String EPHEMERIS_FILE_3    = "C:\\Desktop\\kinematica\\brdm1100.17p.Z";

	private KinematicaApi kinematicaApi = new KinematicaApi();

	public static void main(String[] args) throws Exception
	{
		APIExample example = new APIExample();
		example.run();
	}

	public APIExample()
	{

	}

	public void run() throws Exception
	{
		kinematicaApi = new KinematicaApi();
		kinematicaApi.setApiKey(API_KEY);

		displayDataSets();

		int id = newDataSet();

		editConfiguration(id);

		processDataSet(id);

		// Monitor status
		// Calls finishedProcessing() once data set has finished processing
		dataSetStatusThread(id);

		// Monitor log
		// Will print out log messages
		dataSetLogThread(id);
	}

	private void displayDataSets() throws Exception
	{
		// List the data sets on the account
		ArrayList<DataSetJSON> dataSets = kinematicaApi.allDataSets();

		for (DataSetJSON dataSet : dataSets)
		{
			System.out.print("id:" + dataSet.id + ", name:" + dataSet.name + ", primary:");
			if (!dataSet.primaryFiles.isEmpty()) System.out.println(dataSet.primaryFiles.get(0));
			else System.out.println("null");
		}
	}

	private int newDataSet() throws Exception
	{
		System.out.println("### New Data Set ###");

		DataSet dataSet = new DataSet();

		// Data set names must be unique.
		// Set the data set name to null for automatic unique data set names.
		dataSet.dataSetName = null; // "Unique Dataset Name";

		// Add primary file
		dataSet.primaryFiles.add(new File(PRIMARY_FILE));

		// Set baseStationSource to manual and provide a base station file
		dataSet.baseStationSource = DataSet.BASE_STATION_SOURCE_MANUAL;
		dataSet.baseStationFiles.add(new File(BASE_STATION_FILE_1));

		// Set ephemeris source to supplement and provide ephemeris files
		dataSet.ephemerisSource = DataSet.EPHEMERIS_SOURCE_SUPPLEMENT;
		dataSet.ephemerisFiles.add(new File(EPHEMERIS_FILE_1));
		dataSet.ephemerisFiles.add(new File(EPHEMERIS_FILE_2));
		dataSet.ephemerisFiles.add(new File(EPHEMERIS_FILE_3));

		// Set sendEmail to true to enable email notifications completion.
		dataSet.sendEmail = true;

		// Create the new data set
		DataSetJSON newDataSet = kinematicaApi.newDataSet(dataSet);

		// Print created data set
		System.out.println("id:" + newDataSet.id + ", name:" + newDataSet.name + ", primary:");
		if (!dataSet.primaryFiles.isEmpty()) System.out.println(dataSet.primaryFiles.get(0));
		else System.out.println("null");
		
		return newDataSet.id;
	}

	public void editConfiguration(int id) throws Exception
	{
		System.out.println("### Edit Configuration ###");

		// List the base station antenna options 
		// (this can be called once and stored)
		ArrayList<String> antennaOptions = kinematicaApi.antennaOptions();
		System.out.println("Antenna options:");
		for (int i = 0; i < antennaOptions.size(); i++)
		{
			if (i != 0) System.out.print(", ");
			System.out.print("\"" + antennaOptions.get(i) + "\"");
		}
		System.out.println();

		Configuration configuration              = new Configuration();
		configuration.deviceId                   = DeviceId.SPATIAL_DUAL;
		configuration.baseStationAntenna         = "Automatically select using RINEX file";
		configuration.roverAntenna               = "Automatically select using RINEX file";
		configuration.gnssOffsetSource           = Configuration.MANUAL_SOURCE;
		configuration.gnssOffset                 = new double[] { 1, 2, 3 };
		configuration.odometerOffsetSource       = Configuration.MANUAL_SOURCE;
		configuration.odometerOffset             = new double[] { 4, 5, 6 };
		configuration.dualAntennaOffsetSource    = Configuration.MANUAL_SOURCE;
		configuration.dualAntennaOffsetType      = Configuration.MANUAL_OFFSET;
		configuration.dualAntennaManualOffset    = new double[] { 7, 8, 9 };
		configuration.referencePointOffsetSource = Configuration.MANUAL_SOURCE;
		configuration.referencePointOffset       = new double[] { 10, 11, 12 };

		boolean editConfiguration = kinematicaApi.editConfiguration(id, configuration);

		System.out.println("editConfiguration: " + editConfiguration);
	}

	public void processDataSet(int id) throws Exception
	{
		// Process Data Set
		System.out.println("### Process Data Set ###");
		
		boolean processDataSet = kinematicaApi.processDataSet(id);
		System.out.println("processDataSet: " + processDataSet);
	}

	public static void finishedProcessing(KinematicaApi kinematicaApi, int id) throws Exception
	{
		// Get a list of the data set's files
		ArrayList<KinematicaFileJSON> files = kinematicaApi.dataSetFiles(id);
		File directory = new File(DOWNLOAD_DIRECTORY + File.separator + id);
		
		for (KinematicaFileJSON file : files)
		{
			// For PostProcessed files only, check the file name
			// if(!file.name.startsWith("PostProcessed")) continue;
			
			File downloadedFile = kinematicaApi.downloadDataSetFile(id, file.name, directory);
			System.out.println("Downloaded " + downloadedFile.getName() + " (" + downloadedFile.length() + ")");
		}
	}

	private void dataSetStatusThread(int id)
	{
		(new Thread()
		{
			public void run()
			{
				boolean startedProcessing = false;
				while (true)
				{
					try
					{
						// Get the data set status
						StatusJSON status = kinematicaApi.dataSetStatus(id);
						System.out.println("status: " + status.status + ", currentProgress: " + status.currentProgress + ", totalProgress:" + status.totalProgress + ", processing:" + status.processing);

						if (status.processing) startedProcessing = true;
						
						// Data set has finished processing but due to an error
						// (totalProgress was set back to 0)
						if (startedProcessing && !status.processing && status.totalProgress == 0)
						{
							System.err.println("Processing failed: " + status.status);
							System.exit(0);
						}
						
						// Data set has finished processing successfully
						// (totalProgress reached 100), download the files
						else if (!status.processing && status.totalProgress == 100)
						{
							finishedProcessing(kinematicaApi, id);
							System.exit(0);
						}
						
						Thread.sleep(1000);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		}).start();

	}

	private void dataSetLogThread(int id)
	{
		(new Thread()
		{
			public void run()
			{
				long lastTime = 0;
				while (true)
				{
					try
					{
						// Get all the data set logs after the time provided
						ArrayList<LogJSON> logs = kinematicaApi.dataSetLogsAfterTime(id, lastTime);
						for (LogJSON log : logs)
						{
							System.out.println("log: " + log.message + ", type: " + log.type + ", time:" + log.time);
						}
						
						// Update the last valid log time
						lastTime = (!logs.isEmpty()) ? logs.get(logs.size() - 1).time : lastTime;
						Thread.sleep(1000);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		}).start();
	}
}
